// ----------------------------------------------------------------------------
// * Raima Database Manager
// *
// * Copyright (c) 2012 Raima Inc.,  All rights reserved.
// *
// * Use of this software, whether in source code format, or in executable,
// * binary object code form, is governed by the Raima LICENSE which
// * is fully described in the LICENSE.TXT file, included within this
// * distribution of files.
// * ----------------------------------------------------------------------------

#include <iostream>

// rdm.h is required in all files using the RDM Database
#include "rdm.h"

// obj03_db.h was generated from the DDL file obj03_db.ddl.
#include "obj03_db.h"

// obj03_db_api.h was generated from the DDL file obj03_db.ddl. with 
// the -cpp option
#include "obj03_db_api.h"

// Name space for the RDM C++ interface
using namespace RDM_CPP;

// EXAMPLE - obj03 for C++
// Each time you run this example, a new record is created in the
// database, then all existing records are read from the database
// and printed.

// The DDL (Database Definition Language) is in obj03_db.ddl. The database
// schema is embedded into the application and the database will be created
// the first time the application is run.  Subsequent runs will use the now
// existing database.

// The following RDM C++ methods are illustrated:
//      Db->Open
//      Db->New_recordWithFieldValues
//      Db->Get_records
//      Cursor->AfterLast
//      Cursor->GetFieldValues
//      Cursor++
// They are fully documented in http://docs.raima.com/rdm/.

// Each of these methods can throw and an exception of type rdm_exception

// The actual database is stored in a directory named 'obj03_db' in this
// project directory.

const char *doors_albums[] = 
{
    "The Doors",
    "Strange Days",
    "Waiting for the Sun"
};

const char *stones_albums[] =
{
    "The Rolling Stones",
    "Out of Our Heads",
    "Beggars Banquet",
    "Tattoo You"
};

const char * nirvana_albums[] =
{
    "Bleach",
    "Nevermind",
    "In Utero"
};

int32_t main(void)
{
    Db_obj03_db  db;
    struct artist artist_rec;
    struct album  album_rec;
    Cursor_artist artist_cursor;
    Cursor_album  album_cursor;
    uint16_t      ii;

    // Open the database and in
    try
    {
        db = db.Open("obj03_db", "x");
        db.Initialize();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") opening/initializing obj03_db database" << endl;
    }

    // Create an artist record in the database
    try
    {
        strcpy(artist_rec.name, "The Doors");
        artist_cursor = db.New_artist_recordWithFieldValues(&artist_rec);
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding artist record" << endl;
        return 1;
    }

    // Add and associate album records
    try
    {
        for (ii = 0; ii < RLEN(doors_albums); ii++)
        {
            strcpy(album_rec.title, doors_albums[ii]);
            album_cursor = db.New_album_recordWithFieldValues(&album_rec);
            album_cursor.Connect_artist_album(artist_cursor);
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding album record" << endl;
        return 1;
    }

    // Create an artist record in the database
    try
    {
        strcpy(artist_rec.name, "The Rolling Stones");
        artist_cursor = db.New_artist_recordWithFieldValues(&artist_rec);
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding artist record" << endl;
        return 1;
    }

    // Add and associate album records
    try
    {
        for (ii = 0; ii < RLEN(stones_albums); ii++)
        {
            strcpy(album_rec.title, stones_albums[ii]);
            album_cursor = db.New_album_recordWithFieldValues(&album_rec);
            album_cursor.Connect_artist_album(artist_cursor);
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding album record" << endl;
        return 1;
    }

    // Create an artist record in the database
    try
    {
        strcpy(artist_rec.name, "Nirvana");
        artist_cursor = db.New_artist_recordWithFieldValues(&artist_rec);
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding artist record" << endl;
        return 1;
    }

    // Add and associate album records
    try
    {
        for (ii = 0; ii < RLEN(nirvana_albums); ii++)
        {
            strcpy(album_rec.title, nirvana_albums[ii]);
            album_cursor = db.New_album_recordWithFieldValues(&album_rec);
            album_cursor.Connect_artist_album(artist_cursor);
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding album record" << endl;
        return 1;
    }

    try
    {
        // Loop through each artist record and display the albums they are 
        // associated with
        artist_cursor = db.Get_artist_records();
        while (!artist_cursor.AfterLast())
        {
            artist_cursor.GetFieldValues(&artist_rec);
            cout << endl << artist_rec.name << endl;

            // Get the the albums for this artist
            album_cursor = artist_cursor.Get_artist_album_members();
            while (!album_cursor.AfterLast())
            {
                album_cursor.GetFieldValues(&album_rec);
                cout << '\t' << album_rec.title << endl;
                album_cursor++;
            }
            artist_cursor++;
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
        return 1;
    }
    
    cout << endl << "Press <Enter> to continue . . .";
    getchar();
    return 0;
}
